@@ -1,8 +1,3 @@
-# This library was written by Gary Bernhardt and is licensed under the BSD
-# license. It grew out of an old version of the mock library by Michael Foord
-# available at http://www.voidspace.org.uk/python/mock.html.
-
-
 import sys
 from functools import wraps
 
@@ -96,6 +91,18 @@ class _Patcher:
         setattr(self.module, self.attribute_name, self.original_object)
 
 
+def isolate(object_path):
+    def decorator(fn):
+        module_name, object_name = object_path.rsplit('.', 1)
+        module = sys.modules[module_name]
+        neighbors = set(dir(module)) - set([object_name])
+        for neighbor in neighbors:
+            neighbor_path = '%s.%s' % (module_name, neighbor)
+            fn = patch(neighbor_path)(fn)
+        return fn
+    return decorator
+
+
 def _importer(target):
     components = target.split('.')
     import_path = components.pop(0)
@@ -175,7 +182,21 @@ def returner(return_value):
 
 
 class Dingus(object):
-    def __init__(self, dingus_name=None, full_name=None, **kwargs):
+    @property
+    def __enter__(self):
+        return self._existing_or_new_child('__enter__')
+
+    def __exit__(self, exc_type=None, exc_val=None, exc_tb=None):
+        if exc_type and exc_type not in self.consumed_context_manager_exceptions:
+            return False
+        else:
+            return True
+
+    def __init__(self,
+                 dingus_name=None,
+                 full_name=None,
+                 consumed_context_manager_exceptions=None,
+                 **kwargs):
         self._parent = None
         self.reset()
         name = 'dingus_%i' % id(self) if dingus_name is None else dingus_name
@@ -184,6 +205,8 @@ class Dingus(object):
         self._full_name = full_name
         self.__name__ = name
         self._full_name = full_name
+        self.consumed_context_manager_exceptions = (
+            consumed_context_manager_exceptions or [])
 
         for attr_name, attr_value in kwargs.iteritems():
             if attr_name.endswith('__returns'):
